import asyncio
import json
from typing import Any, Callable, List, Optional
import websockets
from pydantic import BaseModel

class SubscriptionMessage(BaseModel):
    type: str
    data: Any

class WsSubscriptionsClient:
    def __init__(self, base_url: str, endpoint: str = "/ws"):
        self.base_url = base_url.rstrip("/")
        self.endpoint = endpoint
        self.ws: Optional[websockets.WebSocketClientProtocol] = None
        self.callbacks: List[Callable[[Any], None]] = []
        self.subscribed_apps: List[str] = []
        self._running = False

    async def connect(self):
        ws_url = f"ws://{self.base_url.lstrip('http://').lstrip('https://')}{self.endpoint}"
        self.ws = await websockets.connect(ws_url)
        self._running = True
        asyncio.create_task(self._listen())

    async def disconnect(self):
        self._running = False
        if self.ws:
            await self.ws.close()
            self.ws = None

    async def _listen(self):
        if not self.ws:
            raise RuntimeError("WebSocket connection not established")

        while self._running:
            try:
                message = await self.ws.recv()
                data = json.loads(message)
                for callback in self.callbacks:
                    callback(data)
            except websockets.exceptions.ConnectionClosed:
                if self._running:
                    try:
                        await self.connect()
                    except Exception as e:
                        print(f"Failed to reconnect: {e}")
                        self._running = False
                        break
            except Exception as e:
                print(f"Error in WebSocket listener: {e}")
                self._running = False
                break

    def subscribe(self, application_ids: List[str]):
        if not self.ws:
            raise RuntimeError("WebSocket connection not established")

        self.subscribed_apps.extend(application_ids)
        message = {
            "type": "subscribe",
            "data": {
                "applicationIds": application_ids
            }
        }
        asyncio.create_task(self.ws.send(json.dumps(message)))

    def unsubscribe(self, application_ids: List[str]):
        if not self.ws:
            raise RuntimeError("WebSocket connection not established")

        self.subscribed_apps = [app_id for app_id in self.subscribed_apps if app_id not in application_ids]
        message = {
            "type": "unsubscribe",
            "data": {
                "applicationIds": application_ids
            }
        }
        asyncio.create_task(self.ws.send(json.dumps(message)))

    def add_callback(self, callback: Callable[[Any], None]):
        self.callbacks.append(callback)

    def remove_callback(self, callback: Callable[[Any], None]):
        self.callbacks.remove(callback) 


import pytest
import os
from pathlib import Path
import tempfile
import toml
import base58
import nacl.signing
from calimero import Ed25519Keypair, Config

def create_test_keypair():
    """Create a test keypair with proper encoding."""
    signing_key = nacl.signing.SigningKey.generate()
    keypair_bytes = b'\x08\x01\x12\x40' + signing_key.encode() + signing_key.verify_key.encode()
    return base58.b58encode(keypair_bytes).decode()

@pytest.fixture
def temp_config_file():
    """Create a temporary TOML config file."""
    keypair_base58 = create_test_keypair()
    with tempfile.NamedTemporaryFile(mode='w', suffix='.toml', delete=False) as f:
        toml.dump({
            'identity': {
                'keypair': keypair_base58
            },
            'network': {
                'rpc_url': 'http://localhost:2428',
                'context_id': 'test-context',
                'executor_public_key': 'test-executor',
                'node_name': 'test-node'
            }
        }, f)
        return f.name

@pytest.fixture
def ed25519_keypair():
    """Create a test Ed25519 keypair."""
    signing_key = nacl.signing.SigningKey.generate()
    return Ed25519Keypair(signing_key)

@pytest.fixture
def mock_env_vars(monkeypatch):
    """Set up mock environment variables."""
    keypair_base58 = create_test_keypair()
    monkeypatch.setenv('RPC_URL', 'http://localhost:2428')
    monkeypatch.setenv('CONTEXT_ID', 'test-context')
    monkeypatch.setenv('EXECUTOR_PUBLIC_KEY', 'test-executor')
    monkeypatch.setenv('CALIMERO_NODE_NAME', 'test-node')
    monkeypatch.setenv('CALIMERO_KEYPAIR', keypair_base58)
    return monkeypatch

@pytest.fixture
def config(temp_config_file, mock_env_vars):
    """Create a test Config instance."""
    return Config.load_from_file(temp_config_file)

@pytest.fixture
def mock_ws_url():
    """Return a mock WebSocket URL for testing."""
    return "ws://localhost:2428/ws"

@pytest.fixture
def mock_rpc_url():
    """Return a mock RPC URL for testing."""
    return "http://localhost:2428/jsonrpc" 


import base58
from typing import Tuple
import nacl.signing
import nacl.encoding
from typing import Optional

class KeypairError(Exception):
    """Exception raised for keypair-related errors."""
    pass

class Ed25519Keypair:
    """Ed25519 keypair implementation."""
    
    PROTOBUF_PREFIX = b'\x08\x01\x12\x40'  # 4-byte protobuf prefix for Ed25519 keypair

    def __init__(self, signing_key: nacl.signing.SigningKey):
        """Initialize with a PyNaCl signing key."""
        self._signing_key = signing_key
        self._verify_key = signing_key.verify_key

    @classmethod
    def from_base58(cls, base58_keypair: str) -> 'Ed25519Keypair':
        """Create a keypair from a base58-encoded keypair string."""
        if not base58_keypair:
            raise KeypairError("Base58 keypair cannot be None")

        try:
            # Decode base58
            key_bytes = base58.b58decode(base58_keypair)

            # Validate keypair length (should be 68 bytes: 4-byte prefix + 64-byte keypair)
            if len(key_bytes) != 68:
                raise KeypairError(f"Unexpected keypair length: {len(key_bytes)} bytes (expected 68)")

            # Validate protobuf prefix
            prefix = key_bytes[:4]
            if prefix != cls.PROTOBUF_PREFIX:
                raise KeypairError(f"Invalid protobuf prefix: {prefix.hex()} (expected {cls.PROTOBUF_PREFIX.hex()})")

            # Extract private key (first 32 bytes after prefix)
            private_key = key_bytes[4:36]
            if len(private_key) != 32:
                raise KeypairError(f"Invalid private key length: {len(private_key)} (expected 32)")

            # Create signing key from private key
            signing_key = nacl.signing.SigningKey(private_key)
            return cls(signing_key)

        except Exception as e:
            raise KeypairError(f"Failed to create keypair: {str(e)}")

    def to_base58(self) -> str:
        """Convert keypair to base58-encoded string with protobuf prefix."""
        # Get the private key bytes
        private_key = self._signing_key.encode()
        
        # Create the full keypair bytes with protobuf prefix
        keypair_bytes = self.PROTOBUF_PREFIX + private_key
        
        # Encode to base58
        return base58.b58encode(keypair_bytes).decode()

    def sign(self, message: bytes) -> bytes:
        """Sign a message."""
        return self._signing_key.sign(message).signature

    def verify(self, message: bytes, signature: bytes) -> bool:
        """Verify a signature."""
        try:
            self._verify_key.verify(message, signature)
            return True
        except nacl.exceptions.BadSignatureError:
            return False

    def get_public_key(self) -> bytes:
        """Get the public key."""
        return self._verify_key.encode()

    def get_public_key_base58(self) -> str:
        """Get the public key as base58-encoded string."""
        return base58.b58encode(self.get_public_key()).decode()

    @property
    def public_key(self) -> bytes:
        """Get the public key in raw bytes."""
        return self._verify_key.encode()
    
    @property
    def public_key_b58(self) -> str:
        """Get the public key in base58 format."""
        return base58.b58encode(self.public_key).decode()
    
    @property
    def private_key(self) -> bytes:
        """Get the private key."""
        return self._signing_key.encode()

    @property
    def private_key_base58(self) -> str:
        """Get the base58-encoded private key."""
        return base58.b58encode(self.private_key).decode()

    def sign_base58(self, message: str) -> str:
        """Sign a message and return base58-encoded signature."""
        signature = self.sign(message.encode())
        return base58.b58encode(signature).decode()

    def verify_base58(self, message: str, base58_signature: str) -> bool:
        """Verify a base58-encoded signature."""
        signature = base58.b58decode(base58_signature)
        return self.verify(message.encode(), signature) 


import os
from pathlib import Path
from typing import Optional, Dict, Any
import toml
from pydantic import BaseModel, ConfigDict
from .keypair import Ed25519Keypair

class IdentityConfig(BaseModel):
    peer_id: str
    keypair: str

class SwarmConfig(BaseModel):
    listen: list[str]

class ServerConfig(BaseModel):
    listen: list[str]
    admin: Dict[str, bool]
    jsonrpc: Dict[str, bool]
    websocket: Dict[str, bool]

class ConfigError(Exception):
    """Exception raised for configuration-related errors."""
    pass

class Config:
    """Configuration for Calimero client."""
    
    def __init__(self, file_path: str):
        """Initialize with a TOML file path."""
        self._config_data = self._load_toml(file_path)
        
        # Load keypair
        keypair_value = self._config_data.get('identity', {}).get('keypair')
        if not keypair_value:
            raise ConfigError("'keypair' not found in [identity] section")
        self.keypair = Ed25519Keypair.from_base58(keypair_value)
        
        # Load network configuration
        network = self._config_data.get('network', {})
        self.node_url = network.get('rpc_url')
        self.context_id = network.get('context_id')
        self.executor_public_key = network.get('executor_public_key')
        self.node_name = network.get('node_name')
        
        # Validate required fields
        if not self.node_url:
            raise ConfigError("'rpc_url' not found in [network] section")
        if not self.context_id:
            raise ConfigError("'context_id' not found in [network] section")
        if not self.executor_public_key:
            raise ConfigError("'executor_public_key' not found in [network] section")
        if not self.node_name:
            raise ConfigError("'node_name' not found in [network] section")
    
    def __getitem__(self, key: str) -> Any:
        """Allow dynamic access to raw config data."""
        return self._config_data[key]
    
    def __getattr__(self, name: str) -> Any:
        """Allow dynamic access to config sections."""
        if name in self._config_data:
            return self._config_data[name]
        raise AttributeError(f"'Config' object has no attribute '{name}'")
    
    @classmethod
    def load_from_file(cls, file_path: str) -> 'Config':
        """Load configuration from a TOML file."""
        return cls(file_path)
    
    @classmethod
    def get_default_config_path(cls) -> Path:
        """Get the default configuration path."""
        node_name = os.getenv('CALIMERO_NODE_NAME', 'node1')
        return Path.home() / '.calimero' / node_name / 'config.toml'
    
    def _load_toml(self, file_path: str) -> dict:
        """Load and parse TOML file."""
        try:
            with open(file_path, 'r') as f:
                return toml.load(f)
        except Exception as e:
            raise ConfigError(f"Failed to load config file: {str(e)}")
    
    @classmethod
    def load_from_env(cls) -> 'Config':
        """Load configuration from environment variables."""
        import tempfile
        
        # Create temporary TOML file
        with tempfile.NamedTemporaryFile(mode='w', suffix='.toml', delete=False) as f:
            config_data = {
                'identity': {
                    'keypair': os.getenv('CALIMERO_KEYPAIR')
                },
                'network': {
                    'rpc_url': os.getenv('RPC_URL', 'http://localhost:2428'),
                    'context_id': os.getenv('CONTEXT_ID'),
                    'executor_public_key': os.getenv('EXECUTOR_PUBLIC_KEY'),
                    'node_name': os.getenv('CALIMERO_NODE_NAME')
                }
            }
            toml.dump(config_data, f)
            f.flush()
            return cls(f.name) 


import pytest
import os
import tempfile
import toml
import base58
import nacl.signing
from calimero import Config, ConfigError, Ed25519Keypair

def test_config_creation(temp_config_file, mock_env_vars):
    """Test config creation."""
    config = Config.load_from_file(temp_config_file)
    assert config.node_url == "http://localhost:2428"
    assert config.context_id == "test-context"
    assert config.executor_public_key == "test-executor"
    assert config.node_name == "test-node"
    assert isinstance(config.keypair, Ed25519Keypair)

def test_config_from_file(temp_config_file, mock_env_vars):
    """Test loading config from file."""
    config = Config.load_from_file(temp_config_file)
    assert config.node_url == "http://localhost:2428"
    assert config.context_id == "test-context"
    assert config.executor_public_key == "test-executor"
    assert config.node_name == "test-node"
    assert isinstance(config.keypair, Ed25519Keypair)

def test_config_from_env(mock_env_vars, temp_config_file):
    """Test loading config from environment variables."""
    config = Config.load_from_env()
    assert config.node_url == "http://localhost:2428"
    assert config.context_id == "test-context"
    assert config.executor_public_key == "test-executor"
    assert config.node_name == "test-node"
    assert isinstance(config.keypair, Ed25519Keypair)

def test_config_missing_env_vars(monkeypatch):
    """Test config behavior when environment variables are missing."""
    # Clear environment variables
    monkeypatch.delenv('RPC_URL', raising=False)
    monkeypatch.delenv('CONTEXT_ID', raising=False)
    monkeypatch.delenv('EXECUTOR_PUBLIC_KEY', raising=False)
    monkeypatch.delenv('CALIMERO_NODE_NAME', raising=False)
    monkeypatch.delenv('CALIMERO_KEYPAIR', raising=False)

    # Should raise ConfigError when required variables are missing
    with pytest.raises(ConfigError):
        Config.load_from_env()

def test_config_with_keypair(temp_config_file, mock_env_vars):
    """Test config with keypair."""
    config = Config.load_from_file(temp_config_file)
    assert isinstance(config.keypair, Ed25519Keypair)
    assert len(config.keypair.public_key) == 32
    assert len(config.keypair.private_key) == 32

def test_config_missing_keypair():
    """Test config behavior when keypair is missing."""
    with tempfile.NamedTemporaryFile(mode='w', suffix='.toml', delete=False) as f:
        toml.dump({
            'network': {
                'rpc_url': 'http://localhost:2428',
                'context_id': 'test-context',
                'executor_public_key': 'test-executor',
                'node_name': 'test-node'
            }
        }, f)
        f.flush()
        
        with pytest.raises(ConfigError):
            Config.load_from_file(f.name) 


"""
Calimero Network Python Client SDK
"""

__version__ = "0.1.0"

from .config import Config, ConfigError
from .keypair import Ed25519Keypair, KeypairError
from .json_rpc_client import JsonRpcClient
from .ws_subscriptions_client import WsSubscriptionsClient

__all__ = [
    'Config',
    'ConfigError',
    'Ed25519Keypair',
    'KeypairError',
    'JsonRpcClient',
    'WsSubscriptionsClient'
] 


import pytest
import aiohttp
import os
from unittest.mock import Mock, patch
from calimero import JsonRpcClient, Config

@pytest.fixture
def mock_config():
    """Create a mock config for testing."""
    config = Mock(spec=Config)
    config.node_url = "http://localhost:2428"
    config.keypair = Mock()
    config.keypair.sign.return_value = b"signature"
    return config

@pytest.fixture
def mock_env_vars():
    """Set up mock environment variables."""
    with patch.dict(os.environ, {
        'CONTEXT_ID': 'test_context',
        'EXECUTOR_PUBLIC_KEY': 'test_key'
    }):
        yield

@pytest.mark.asyncio
async def test_json_rpc_client_creation(mock_config, mock_env_vars):
    """Test JsonRpcClient initialization."""
    client = JsonRpcClient(mock_config)
    assert client.node_url == "http://localhost:2428"
    assert client.context_id == "test_context"
    assert client.executor_public_key == "test_key"

@pytest.mark.asyncio
async def test_prepare_headers(mock_config, mock_env_vars):
    """Test header preparation."""
    client = JsonRpcClient(mock_config)
    headers = client._prepare_headers()
    
    assert 'Content-Type' in headers
    assert 'X-Signature' in headers
    assert 'X-Timestamp' in headers
    assert headers['Content-Type'] == 'application/json'

@pytest.mark.asyncio
async def test_prepare_request(mock_config, mock_env_vars):
    """Test request preparation."""
    client = JsonRpcClient(mock_config)
    request = client._prepare_request("test_method", {"arg": "value"})
    
    assert request['jsonrpc'] == '2.0'
    assert request['method'] == 'execute'
    assert request['params']['contextId'] == 'test_context'
    assert request['params']['method'] == 'test_method'
    assert request['params']['argsJson'] == {"arg": "value"}
    assert request['params']['executorPublicKey'] == 'test_key'

@pytest.mark.asyncio
async def test_execute_request(mock_config, mock_env_vars):
    """Test executing RPC request."""
    client = JsonRpcClient(mock_config)
    
    mock_response = {
        "jsonrpc": "2.0",
        "result": {"success": True},
        "id": 1
    }
    
    with patch('aiohttp.ClientSession.post') as mock_post:
        mock_post.return_value.__aenter__.return_value.json.return_value = mock_response
        mock_post.return_value.__aenter__.return_value.status = 200
        
        response = await client.execute("test_method", {"arg": "value"})
        assert response == mock_response
        
        # Verify request was made with correct parameters
        mock_post.assert_called_once()
        call_args = mock_post.call_args[1]
        assert "json" in call_args
        assert call_args["json"]["method"] == "execute"
        assert call_args["json"]["params"]["contextId"] == "test_context"

@pytest.mark.asyncio
async def test_execute_request_error(mock_config, mock_env_vars):
    """Test error handling in RPC request."""
    client = JsonRpcClient(mock_config)
    
    with patch('aiohttp.ClientSession.post') as mock_post:
        mock_post.return_value.__aenter__.return_value.status = 400
        mock_post.return_value.__aenter__.return_value.text.return_value = "Error message"
        
        with pytest.raises(Exception) as exc_info:
            await client.execute("test_method", {"arg": "value"})
        assert "HTTP error 400" in str(exc_info.value) 


import pytest
import json
from unittest.mock import Mock, patch, AsyncMock
from calimero import WsSubscriptionsClient
import websockets
import asyncio

@pytest.fixture
def mock_ws():
    """Create a mock WebSocket connection."""
    ws = AsyncMock()
    ws.recv.return_value = json.dumps({"type": "test", "data": "test_data"})
    return ws

@pytest.mark.asyncio
async def test_ws_client_creation():
    """Test WsSubscriptionsClient initialization."""
    client = WsSubscriptionsClient("http://localhost:2428")
    assert client.base_url == "http://localhost:2428"
    assert client.endpoint == "/ws"
    assert not client._running
    assert len(client.callbacks) == 0
    assert len(client.subscribed_apps) == 0
    assert client.ws is None

@pytest.mark.asyncio
async def test_connect_and_disconnect():
    """Test WebSocket connection and disconnection."""
    client = WsSubscriptionsClient("http://localhost:2428")
    
    with patch('websockets.connect', new_callable=AsyncMock) as mock_connect:
        mock_ws = AsyncMock()
        mock_connect.return_value = mock_ws
        
        await client.connect()
        assert client._running
        assert client.ws == mock_ws
        
        await client.disconnect()
        assert not client._running
        assert client.ws is None
        mock_ws.close.assert_called_once()

@pytest.mark.asyncio
async def test_subscribe_and_unsubscribe():
    """Test subscription management."""
    client = WsSubscriptionsClient("http://localhost:2428")
    client.ws = AsyncMock()
    
    # Test subscribe
    app_ids = ["app1", "app2"]
    client.subscribe(app_ids)
    assert client.subscribed_apps == app_ids
    
    # Verify subscribe message was sent
    expected_message = {
        "type": "subscribe",
        "data": {
            "applicationIds": app_ids
        }
    }
    client.ws.send.assert_called_once_with(json.dumps(expected_message))
    
    # Test unsubscribe
    client.unsubscribe(["app1"])
    assert client.subscribed_apps == ["app2"]
    
    # Verify unsubscribe message was sent
    expected_message = {
        "type": "unsubscribe",
        "data": {
            "applicationIds": ["app1"]
        }
    }
    assert client.ws.send.call_count == 2
    client.ws.send.assert_called_with(json.dumps(expected_message))

@pytest.mark.asyncio
async def test_callback_management():
    """Test callback management."""
    client = WsSubscriptionsClient("http://localhost:2428")
    
    # Test adding callback
    callback1 = Mock()
    callback2 = Mock()
    client.add_callback(callback1)
    client.add_callback(callback2)
    assert len(client.callbacks) == 2
    
    # Test removing callback
    client.remove_callback(callback1)
    assert len(client.callbacks) == 1
    assert client.callbacks[0] == callback2 


import asyncio
import os
import time
import base58
import json
from typing import Optional, Dict, Any, TypedDict
import aiohttp
from .config import Config
from .keypair import Ed25519Keypair

class JsonRpcError(Exception):
    """Base exception for JSON-RPC errors."""
    pass

class JsonRpcResponse(TypedDict):
    """Type definition for JSON-RPC response."""
    jsonrpc: str
    id: int
    result: Optional[Dict[str, Any]]
    error: Optional[Dict[str, Any]]

class JsonRpcClient:
    """JSON-RPC client for Calimero.
    
    This client handles communication with the Calimero JSON-RPC server,
    including request formatting, signing, and response handling.
    """
    
    # Constants
    JSONRPC_VERSION = '2.0'
    DEFAULT_TIMEOUT = 1000
    JSONRPC_PATH = '/jsonrpc/dev'
    
    def __init__(self, config: Config):
        """Initialize the JSON-RPC client with a config.
        
        Args:
            config: Configuration object containing node URL and keypair.
            
        Raises:
            ValueError: If required environment variables are not set.
        """
        self.config = config
        self.node_url = config.node_url
        self.keypair = config.keypair
        self.context_id = os.getenv('CONTEXT_ID')
        self.executor_public_key = os.getenv('EXECUTOR_PUBLIC_KEY')
        
        self._validate_environment()
    
    def _validate_environment(self) -> None:
        """Validate that required environment variables are set.
        
        Raises:
            ValueError: If any required environment variable is missing.
        """
        if not self.context_id:
            raise ValueError("CONTEXT_ID environment variable is not set")
        if not self.executor_public_key:
            raise ValueError("EXECUTOR_PUBLIC_KEY environment variable is not set")
    
    def _prepare_headers(self) -> Dict[str, str]:
        """Prepare request headers with signature and timestamp.
        
        Returns:
            Dictionary containing request headers.
        """
        timestamp = str(int(time.time()))
        signature = self.keypair.sign(timestamp.encode())
        signature_b58 = base58.b58encode(signature).decode()
        
        return {
            'Content-Type': 'application/json',
            'X-Signature': signature_b58,
            'X-Timestamp': timestamp
        }
    
    def _prepare_request(self, method: str, args: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """Prepare the JSON-RPC request payload.
        
        Args:
            method: The method to call.
            args: Optional arguments for the method.
            
        Returns:
            Dictionary containing the JSON-RPC request payload.
        """
        return {
            'jsonrpc': self.JSONRPC_VERSION,
            'id': 1,
            'method': 'execute',
            'params': {
                'contextId': self.context_id,
                'method': method,
                'argsJson': args or {},
                'executorPublicKey': self.executor_public_key,
                'timeout': self.DEFAULT_TIMEOUT
            }
        }
    
    async def _handle_response(self, response: aiohttp.ClientResponse) -> JsonRpcResponse:
        """Handle the JSON-RPC response.
        
        Args:
            response: The HTTP response from the server.
            
        Returns:
            Parsed JSON-RPC response.
            
        Raises:
            JsonRpcError: If the response indicates an error.
        """
        if response.status != 200:
            error_text = await response.text()
            raise JsonRpcError(f"HTTP error {response.status}: {error_text}")
        
        result = await response.json()
        if os.getenv('VERBOSE') == '1':
            print(f"Result: {json.dumps(result, indent=2)}")
            
        if 'error' in result:
            raise JsonRpcError(f"JSON-RPC error: {result['error']}")
            
        return result
    
    async def execute(self, method: str, args: Optional[Dict[str, Any]] = None) -> JsonRpcResponse:
        """Execute a JSON-RPC request.
        
        Args:
            method: The method to call.
            args: Optional arguments for the method.
            
        Returns:
            Parsed JSON-RPC response.
            
        Raises:
            JsonRpcError: If the request fails or returns an error.
        """
        headers = self._prepare_headers()
        payload = self._prepare_request(method, args)
        
        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    f"{self.node_url}{self.JSONRPC_PATH}",
                    headers=headers,
                    json=payload
                ) as response:
                    return await self._handle_response(response)
        except aiohttp.ClientError as e:
            raise JsonRpcError(f"Network error: {str(e)}")
        except json.JSONDecodeError as e:
            raise JsonRpcError(f"Invalid JSON response: {str(e)}") 


from setuptools import setup, find_packages

setup(
    name="calimero-client-py",
    version="0.1.1",
    packages=find_packages(),
    install_requires=[
        "requests>=2.31.0",
        "websockets>=12.0",
        "base58>=2.1.1",
        "pydantic>=2.5.0",
        "aiohttp>=3.9.0",
        "toml>=0.10.2",
        "pynacl>=1.5.0",
    ],
    extras_require={
        "test": [
            "pytest>=7.0.0",
            "pytest-asyncio>=0.26.0",
            "pytest-cov>=4.0.0",
            "pytest-mock>=3.10.0",
        ],
    },
    author="Calimero Network",
    author_email="support@calimero.network",
    description="Python client SDK for Calimero Network",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/calimero-network/calimero-client-py",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    python_requires=">=3.8",
) 


import pytest
import nacl.signing
import base58
from calimero import Ed25519Keypair, KeypairError

def test_keypair_creation():
    """Test creating a keypair."""
    # Generate a new keypair
    signing_key = nacl.signing.SigningKey.generate()
    keypair = Ed25519Keypair(signing_key)
    
    # Verify public key
    assert keypair.public_key == signing_key.verify_key.encode()
    assert keypair.public_key_b58 == base58.b58encode(signing_key.verify_key.encode()).decode()

def test_keypair_from_base58():
    """Test creating keypair from base58-encoded keypair with protobuf prefix."""
    # Generate a new keypair
    signing_key = nacl.signing.SigningKey.generate()
    
    # Create keypair bytes with protobuf prefix
    keypair_bytes = b'\x08\x01\x12\x40' + signing_key.encode() + signing_key.verify_key.encode()
    keypair_base58 = base58.b58encode(keypair_bytes).decode()
    
    # Create keypair from base58
    keypair = Ed25519Keypair.from_base58(keypair_base58)
    
    # Verify public key
    assert keypair.public_key == signing_key.verify_key.encode()
    assert keypair.public_key_b58 == base58.b58encode(signing_key.verify_key.encode()).decode()

def test_sign_and_verify():
    """Test signing and verifying messages."""
    # Generate a new keypair
    signing_key = nacl.signing.SigningKey.generate()
    keypair = Ed25519Keypair(signing_key)
    
    # Sign a message
    message = b"test message"
    signature = keypair.sign(message)
    
    # Verify the signature
    assert keypair.verify(message, signature)
    
    # Verify with wrong message
    assert not keypair.verify(b"wrong message", signature)

def test_sign_and_verify_base58():
    """Test signing and verifying messages with base58 encoding."""
    # Generate a new keypair
    signing_key = nacl.signing.SigningKey.generate()
    keypair = Ed25519Keypair(signing_key)
    
    # Sign a message
    message = "test message"
    signature_base58 = keypair.sign_base58(message)
    
    # Verify the signature
    assert keypair.verify_base58(message, signature_base58)
    
    # Verify with wrong message
    assert not keypair.verify_base58("wrong message", signature_base58)

def test_invalid_base58():
    """Test handling of invalid base58 string."""
    with pytest.raises(KeypairError):
        Ed25519Keypair.from_base58("")

def test_invalid_keypair_length():
    """Test handling of invalid keypair length."""
    # Generate a keypair with wrong length
    signing_key = nacl.signing.SigningKey.generate()
    keypair_bytes = b'\x08\x01\x12\x40' + signing_key.encode()  # Missing verify key
    keypair_base58 = base58.b58encode(keypair_bytes).decode()
    
    with pytest.raises(KeypairError):
        Ed25519Keypair.from_base58(keypair_base58)

def test_invalid_protobuf_prefix():
    """Test handling of invalid protobuf prefix."""
    # Generate a keypair with wrong prefix
    signing_key = nacl.signing.SigningKey.generate()
    keypair_bytes = b'\x00\x00\x00\x00' + signing_key.encode() + signing_key.verify_key.encode()
    keypair_base58 = base58.b58encode(keypair_bytes).decode()
    
    with pytest.raises(KeypairError):
        Ed25519Keypair.from_base58(keypair_base58) 